(function() {
  'use strict';
  var $window, InitSelectM, SelectM;

  $window = $(window);

  SelectM = function($wrapper) {
    this.$wrapper = $wrapper;
    this.$labelEl = this.$wrapper.find('label');
    this.$selectEl = this.$wrapper.find('select');
    this.$optionEls = this.$selectEl.children('option');
    this.createHtml();
    this.initToggle();
    this.initKeyboardToggle();
    this.initChoose();
    return this;
  };

  SelectM.prototype.createHtml = function() {
    var initListPoint;
    this.isMultiple = false;
    this.isDisabled = false;
    this.$labelEl.attr('tabindex', '-1');
    this.$selectEl.attr('tabindex', '-1');
    this.$selectM = $('<div class="select-m"></div>');
    if (this.$selectEl.prop('multiple')) {
      this.$selectM.addClass('multiple');
      this.isMultiple = true;
    }
    if (this.$selectEl.prop('disabled')) {
      this.$selectM.addClass('disabled');
      this.isDisabled = true;
    }
    if (this.isMultiple === true) {
      this.selectedValues = [];
    } else {
      this.selectedValues = null;
    }
    this.$toggle = $('<a class="select-m-toggle" href="javascript://open-list" draggable="false"></a>');
    this.$toggleText = $('<span class="select-m-toggle__text"></span>');
    this.$toggleIcon = $('<i class="select-m-toggle__icon"></i>');
    this.$toggleText.appendTo(this.$toggle);
    this.$toggleIcon.appendTo(this.$toggle);
    this.$toggle.appendTo(this.$selectM);
    this.$list = $('<ul class="select-m-list" tabindex="-1"></ul>');
    this.$points = null;
    initListPoint = function(index, element) {
      var $optionEl, $point, text;
      $optionEl = $(element);
      text = $optionEl.text();
      $point = $('<li class="select-m-list__point"></li>');
      $point.data('option', $optionEl);
      $point.text(text);
      if ($optionEl.prop('selected') === true) {
        if (this.isMultiple === true) {
          this.selectedValues.push(text);
        } else {
          this.selectedValues = text;
        }
        $point.addClass('selected');
      }
      if ($optionEl.prop('disabled') === true) {
        $point.addClass('disabled');
      }
      if (this.$points === null) {
        this.$points = $point;
      } else {
        this.$points = this.$points.add($point);
      }
      $point.appendTo(this.$list);
      return this;
    };
    this.$optionEls.each($.proxy(initListPoint, this));
    this.$list.appendTo(this.$selectM);
    this.updateToggleText();
    this.$selectM.appendTo(this.$wrapper);
    return this;
  };

  SelectM.prototype.updateToggleText = function() {
    var text;
    text = null;
    if ((this.selectedValues !== null) && (this.selectedValues.length > 0)) {
      if (this.isMultiple === true) {
        text = this.selectedValues.join(', ');
      } else {
        text = this.selectedValues;
      }
    } else {
      text = 'Nothing is chosen';
    }
    this.$toggleText.text(text);
    return this;
  };

  SelectM.prototype.initToggle = function() {
    var focused, toggle;
    this.$selectM.on('click', function(e) {
      return e.stopPropagation();
    });
    this.isActive = false;
    this.open = function() {
      $window.trigger('close.slm-toggle');
      this.$selectM.addClass('active');
      this.$list.attr('tabindex', '0');
      this.$list.trigger('focus');
      this.isActive = true;
      $window.on('click.slm-toggle close.slm-toggle', $.proxy(this.close, this));
      return this;
    };
    this.close = function() {
      this.$list.trigger('blur');
      this.$list.attr('tabindex', '-1');
      this.$selectM.removeClass('active');
      if (typeof this.clearFocus === 'function') {
        this.clearFocus();
      }
      this.isActive = false;
      $window.off('click.slm-toggle close.slm-toggle');
      return this;
    };
    toggle = function(e) {
      e.preventDefault();
      this.$toggle.trigger('blur');
      if (this.isDisabled === false) {
        if (this.isActive === false) {
          this.open();
        } else {
          this.close();
        }
      }
      return this;
    };
    this.$toggle.on('click', $.proxy(toggle, this));
    this.currentFocusedPoint = -1;
    focused = function(e) {
      var $currentFocusedPoint;
      $currentFocusedPoint = $(e.target);
      this.currentFocusedPoint = this.$points.index($currentFocusedPoint);
      if ($currentFocusedPoint.hasClass('disabled') === false) {
        $currentFocusedPoint.addClass('focused').siblings().removeClass('focused');
      }
      return this;
    };
    this.$list.on('mousemove', '.select-m-list__point', $.proxy(focused, this));
    this.clearFocus = function() {
      this.$points.removeClass('focused');
      this.currentFocusedPoint = -1;
      return this;
    };
    return this;
  };

  SelectM.prototype.initKeyboardToggle = function() {
    var keyboardToggle;
    this.checkScrollTop = function($point) {
      var currentScrollTop, listHeight, listOuter, listOuterHeight, pointBottom, pointHeight, pointTop;
      listHeight = this.$list.height();
      listOuterHeight = this.$list.outerHeight();
      listOuter = listOuterHeight - listHeight;
      pointHeight = $point.outerHeight();
      pointTop = $point.position().top;
      pointBottom = pointTop + pointHeight;
      currentScrollTop = this.$list.scrollTop();
      if (pointBottom > listOuterHeight) {
        this.$list.scrollTop(pointBottom - listOuterHeight + currentScrollTop + listOuter);
      }
      if (pointTop < 0) {
        this.$list.scrollTop(currentScrollTop - Math.abs(pointTop));
      }
      return this;
    };
    this.moveFocusDown = function() {
      var $currentFocusedPoint;
      if ((++this.currentFocusedPoint) === this.$points.length) {
        this.currentFocusedPoint = 0;
      }
      $currentFocusedPoint = this.$points.eq(this.currentFocusedPoint);
      if ($currentFocusedPoint.hasClass('disabled') === true) {
        this.moveFocusDown();
      } else {
        $currentFocusedPoint.addClass('focused').siblings().removeClass('focused');
        this.checkScrollTop($currentFocusedPoint);
      }
      return this;
    };
    this.moveFocusUp = function() {
      var $currentFocusedPoint;
      if ((--this.currentFocusedPoint) < 0) {
        this.currentFocusedPoint = this.$points.length - 1;
      }
      $currentFocusedPoint = this.$points.eq(this.currentFocusedPoint);
      if ($currentFocusedPoint.hasClass('disabled') === true) {
        this.moveFocusUp();
      } else {
        $currentFocusedPoint.addClass('focused').siblings().removeClass('focused');
        this.checkScrollTop($currentFocusedPoint);
      }
      return this;
    };
    this.selectFocus = function() {
      var $currentFocusedPoint;
      if (this.currentFocusedPoint >= 0) {
        $currentFocusedPoint = this.$points.eq(this.currentFocusedPoint);
        $currentFocusedPoint.trigger('click');
      }
      return this;
    };
    keyboardToggle = function(e) {
      if (e.keyCode === 40) {
        e.preventDefault();
        this.moveFocusDown();
      }
      if (e.keyCode === 38) {
        e.preventDefault();
        this.moveFocusUp();
      }
      if (e.keyCode === 13 || e.keyCode === 32) {
        e.preventDefault();
        this.selectFocus();
      }
      if (e.keyCode === 27) {
        e.preventDefault();
        this.close();
      }
      return this;
    };
    this.$list.on('keydown', $.proxy(keyboardToggle, this));
    return this;
  };

  SelectM.prototype.initChoose = function() {
    var choose;
    choose = function(e) {
      var $currentOption, $currentPoint, $otherOptions, $otherPoints, indexOf;
      $currentPoint = this.$points.filter(e.target);
      $currentOption = $currentPoint.data('option');
      if ($currentPoint.hasClass('disabled') === false) {
        if (this.isMultiple === true) {
          if ($currentPoint.hasClass('selected') === false) {
            $currentPoint.addClass('selected');
            $currentOption.prop('selected', true);
            this.selectedValues.push($currentPoint.text());
          } else {
            $currentPoint.removeClass('selected');
            $currentOption.prop('selected', false);
            indexOf = $.inArray($currentPoint.text(), this.selectedValues);
            this.selectedValues.splice(indexOf, 1);
          }
          this.$selectEl.trigger('change');
        } else {
          $otherPoints = this.$points.not($currentPoint);
          $otherOptions = this.$optionEls.not($currentOption);
          if ($currentPoint.hasClass('selected') === false) {
            $currentPoint.addClass('selected');
            $currentOption.prop('selected', true);
            $otherPoints.removeClass('selected');
            $otherOptions.prop('selected', false);
            this.$selectEl.trigger('change');
            this.selectedValues = $currentPoint.text();
          }
          this.close();
        }
        this.updateToggleText();
      }
      return this;
    };
    this.$list.on('click', 'li', $.proxy(choose, this));
    return this;
  };

  InitSelectM = function() {
    var $selectsWrappers, initSelectM;
    this.selectMArr = [];
    initSelectM = function(index, element) {
      var $selectWrapper, selectM;
      $selectWrapper = $(element);
      selectM = new SelectM($selectWrapper);
      this.selectMArr.push(selectM);
      return this;
    };
    $selectsWrappers = $('[data-slm]');
    $selectsWrappers.each($.proxy(initSelectM, this));
    return this.selectMArr;
  };

  this.selectM = new InitSelectM();

}).call(this);
