'use strict'


$window = $ window


SelectM = ($wrapper) ->
  this.$wrapper = $wrapper
  this.$labelEl = this.$wrapper.find 'label'
  this.$selectEl = this.$wrapper.find 'select'
  this.$optionEls = this.$selectEl.children 'option'

  this.createHtml()
  this.initToggle()
  this.initKeyboardToggle()
  this.initChoose()

  return this


SelectM.prototype.createHtml = ->
  this.isMultiple = false
  this.isDisabled = false

  this.$labelEl.attr 'tabindex', '-1'
  this.$selectEl.attr 'tabindex', '-1'

  this.$selectM = $ '<div class="select-m"></div>'

  if this.$selectEl.prop 'multiple'
    this.$selectM.addClass 'multiple'
    this.isMultiple = true

  if this.$selectEl.prop 'disabled'
    this.$selectM.addClass 'disabled'
    this.isDisabled = true

  if this.isMultiple is true
    this.selectedValues = []
  else
    this.selectedValues = null

  this.$toggle = $ '<a class="select-m-toggle" href="javascript://open-list" draggable="false"></a>'
  this.$toggleText = $ '<span class="select-m-toggle__text"></span>'
  this.$toggleIcon = $ '<i class="select-m-toggle__icon"></i>'
  this.$toggleText.appendTo this.$toggle
  this.$toggleIcon.appendTo this.$toggle
  this.$toggle.appendTo this.$selectM

  this.$list = $ '<ul class="select-m-list" tabindex="-1"></ul>'
  this.$points = null

  initListPoint = (index, element) ->
    $optionEl = $ element

    text = $optionEl.text()

    $point = $ '<li class="select-m-list__point"></li>'
    $point.data 'option', $optionEl
    $point.text text

    if $optionEl.prop('selected') is true
      if this.isMultiple is true
        this.selectedValues.push text
      else
        this.selectedValues = text

      $point.addClass 'selected'

    if $optionEl.prop('disabled') is true
      $point.addClass 'disabled'

    if this.$points is null
      this.$points = $point
    else
      this.$points = this.$points.add $point

    $point.appendTo this.$list

    return this

  this.$optionEls.each $.proxy initListPoint, this

  this.$list.appendTo this.$selectM

  this.updateToggleText()

  this.$selectM.appendTo this.$wrapper

  return this


SelectM.prototype.updateToggleText = ->
  text = null

  if (this.selectedValues isnt null) and (this.selectedValues.length > 0)
    if this.isMultiple is true
      text = this.selectedValues.join ', '
    else
      text = this.selectedValues
  else
    text = 'Nothing is chosen'

  this.$toggleText.text text

  return this


SelectM.prototype.initToggle = ->
  this.$selectM.on 'click', (e) -> e.stopPropagation()

  this.isActive = false

  this.open = ->
    $window.trigger 'close.slm-toggle'

    this.$selectM.addClass 'active'
    this.$list.attr 'tabindex', '0'
    this.$list.trigger 'focus'
    this.isActive = true

    $window.on 'click.slm-toggle close.slm-toggle', $.proxy this.close, this

    return this

  this.close = ->
    this.$list.trigger 'blur'
    this.$list.attr 'tabindex', '-1'
    this.$selectM.removeClass 'active'
    this.clearFocus() if typeof this.clearFocus is 'function'
    this.isActive = false

    $window.off 'click.slm-toggle close.slm-toggle'

    return this

  toggle = (e) ->
    e.preventDefault()

    this.$toggle.trigger 'blur'

    if this.isDisabled is false
      if this.isActive is false
        this.open()

      else
        this.close()

    return this

  this.$toggle.on 'click', $.proxy toggle, this

  this.currentFocusedPoint = -1

  focused = (e) ->
    $currentFocusedPoint = $ e.target

    this.currentFocusedPoint = this.$points.index $currentFocusedPoint

    if $currentFocusedPoint.hasClass('disabled') is false
      $currentFocusedPoint.addClass('focused').siblings().removeClass('focused')

    return this

  this.$list.on 'mousemove', '.select-m-list__point', $.proxy focused, this

  this.clearFocus = ->
    this.$points.removeClass 'focused'
    this.currentFocusedPoint = -1

    return this

  return this


SelectM.prototype.initKeyboardToggle = ->
  this.checkScrollTop = ($point) ->
    listHeight = this.$list.height()
    listOuterHeight = this.$list.outerHeight()
    listOuter = listOuterHeight - listHeight

    pointHeight = $point.outerHeight()
    pointTop = $point.position().top
    pointBottom = pointTop + pointHeight

    currentScrollTop = this.$list.scrollTop()

    if pointBottom > listOuterHeight
      this.$list.scrollTop(pointBottom - listOuterHeight + currentScrollTop + listOuter)

    if pointTop < 0
      this.$list.scrollTop(currentScrollTop - Math.abs(pointTop))

    return this

  this.moveFocusDown = ->
    if (++this.currentFocusedPoint) is this.$points.length
      this.currentFocusedPoint = 0

    $currentFocusedPoint = this.$points.eq this.currentFocusedPoint

    if $currentFocusedPoint.hasClass('disabled') is true
      this.moveFocusDown()
    else
      $currentFocusedPoint.addClass('focused').siblings().removeClass('focused')

      this.checkScrollTop $currentFocusedPoint

    return this

  this.moveFocusUp = ->
    if (--this.currentFocusedPoint) < 0
      this.currentFocusedPoint = this.$points.length - 1

    $currentFocusedPoint = this.$points.eq this.currentFocusedPoint

    if $currentFocusedPoint.hasClass('disabled') is true
      this.moveFocusUp()
    else
      $currentFocusedPoint.addClass('focused').siblings().removeClass('focused')

      this.checkScrollTop $currentFocusedPoint

    return this

  this.selectFocus = ->
    if this.currentFocusedPoint >= 0
      $currentFocusedPoint = this.$points.eq this.currentFocusedPoint
      $currentFocusedPoint.trigger 'click'

    return this

  keyboardToggle = (e) ->
    if e.keyCode is 40
      e.preventDefault()
      this.moveFocusDown()

    if e.keyCode is 38
      e.preventDefault()
      this.moveFocusUp()

    if e.keyCode is 13 or e.keyCode is 32
      e.preventDefault()
      this.selectFocus()

    if e.keyCode is 27
      e.preventDefault()
      this.close()

    return this

  this.$list.on 'keydown', $.proxy keyboardToggle, this

  return this


SelectM.prototype.initChoose = ->
  choose = (e) ->
    $currentPoint = this.$points.filter e.target
    $currentOption = $currentPoint.data 'option'

    if $currentPoint.hasClass('disabled') is false
      if this.isMultiple is true
        if $currentPoint.hasClass('selected') is false
          $currentPoint.addClass 'selected'
          $currentOption.prop 'selected', true

          this.selectedValues.push $currentPoint.text()

        else
          $currentPoint.removeClass 'selected'
          $currentOption.prop 'selected', false

          indexOf = $.inArray $currentPoint.text(), this.selectedValues

          this.selectedValues.splice indexOf , 1

        this.$selectEl.trigger 'change'

      else
        $otherPoints = this.$points.not $currentPoint
        $otherOptions = this.$optionEls.not $currentOption

        if $currentPoint.hasClass('selected') is false
          $currentPoint.addClass 'selected'
          $currentOption.prop 'selected', true

          $otherPoints.removeClass 'selected'
          $otherOptions.prop 'selected', false

          this.$selectEl.trigger 'change'
          this.selectedValues = $currentPoint.text()

        this.close()

      this.updateToggleText()

    return this

  this.$list.on 'click', 'li', $.proxy choose, this

  return this




InitSelectM = ->
  this.selectMArr = []

  initSelectM = (index, element) ->
    $selectWrapper = $ element

    selectM = new SelectM $selectWrapper

    this.selectMArr.push selectM

    return this

  $selectsWrappers = $ '[data-slm]'
  $selectsWrappers.each $.proxy initSelectM, this

  return this.selectMArr




this.selectM = new InitSelectM()
